module Benchmark.AMR exposing (suite)

import AMR.Main as AMR
import Benchmark exposing (Benchmark, benchmark)
import Example.Counter as Example
import Help
import Json.Decode


suite : Benchmark
suite =
    Benchmark.describe "AMR"
        [ initWithoutCache
        , updateFirstMsg
        , encodeInitialState
        , decodeInitialState
        ]


initWithoutCache : Benchmark
initWithoutCache =
    Benchmark.compare "init without cache"
        "with AMR"
        (\() ->
            AMR.init (Help.replayUpdate Example.update)
                Example.msgDecoder
                Nothing
                (Example.init ())
        )
        "without"
        (\() -> Example.init)


updateFirstMsg : Benchmark
updateFirstMsg =
    let
        ( model, _ ) =
            AMR.init (Help.replayUpdate Example.update)
                Example.msgDecoder
                Nothing
                (Example.init ())
    in
    Benchmark.compare "update first msg"
        "with AMR"
        (\() ->
            AMR.update Example.update
                (always Cmd.none)
                Example.encodeMsg
                (Example.msgFromInt 0)
                model
        )
        "without"
        (\() ->
            Example.update
                (Example.msgFromInt 0)
                0
        )


encodeInitialState : Benchmark
encodeInitialState =
    let
        ( model, _ ) =
            AMR.init (Help.replayUpdate Example.update)
                Example.msgDecoder
                Nothing
                (Example.init ())
    in
    benchmark "encode initial state"
        (\() -> AMR.encode model)


decodeInitialState : Benchmark
decodeInitialState =
    let
        encoded =
            (AMR.encode << Tuple.first)
                (AMR.init (Help.replayUpdate Example.update)
                    Example.msgDecoder
                    Nothing
                    (Example.init ())
                )
    in
    benchmark "decode initial state"
        (\() ->
            Json.Decode.decodeValue
                (AMR.decoder
                    (Help.replayUpdate Example.update)
                    Example.msgDecoder
                    0
                )
                encoded
        )
